//
// stund/stunq.cxx
// 
// Copyright (c) 2002 Alan Hawrylyshen
//
/// @author Alan Hawrylyshen
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//


#ifndef WIN32
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <errno.h>
#endif



#include <iostream.h>


#include "udp.h"
#include "stun.h"
#include "util.h"
 
#include "assert.h"

/**
 * usage - print usage message
 */
int usage()
{
   cerr << "usage: stunq [-v|--verbose] host[:port]" << endl;
   return -1;
}


/**
  stunq - STUN Query client.
  @param -v Verbose.
 
  OR

  @param --verbose Verbose

 */
int
main(int argc, char* argv[])
{
    assert( sizeof(UInt8 ) == 1 );
    assert( sizeof(UInt16) == 2 );
    assert( sizeof(UInt32) == 4 );
    
    initNetwork();
    
    bool verbose = false;

    // Proc args
    if (argc != 2 && argc != 3)
    {
       return usage();
    }

    int hind = 1;

    if (argc == 3)
    {
       char *p = argv[1];
       if (*p == '-' && *(p+1) == '-') p++;
       verbose = (*p++ == '-' && *p++ == 'v');
       if (!verbose) return usage();
       hind = 2;
    }
    
    StunAddress stunServerAddr;
    stunParseServerName( argv[hind], &stunServerAddr);

    if (verbose) 
    {
       clog << "STUN test with host " << stunServerAddr << endl;
    }
    
    StunNatType stype = stunType( stunServerAddr, verbose );

    clog << "Internet connection is type: ";

    switch (stype)
    {
       case StunTypeOpen:
          clog << "Open";
          break;
       case StunTypeConeNat:
          clog << "Type Cone Nat";
          break;
       case StunTypeRestrictedNat:
          clog << "Restricted Nat";
          break;
       case StunTypePortRestrictedNat:
          clog << "Port Restricted Nat";
          break;
       case StunTypeSymNat:
          clog << "Symetric Nat";
          break;
       case StunTypeSymFirewall:
          clog << "Symetric Firewall";
          break;
       case StunTypeBlocked:
          clog << "Blocked";
          break;
       default:
          clog << stype;
          break;
    }
    clog << endl;

#if 0
    else if ( testNum == -1 )
    {
	StunAddress fd1Addr;

	int fd1 = stunOpenSocket( stunServerAddr, &fd1Addr );

	clog << "Opened a stun socket at " << fd1Addr << endl;
	
	StunAddress fd2Addr;

	int fd2 = stunOpenSocket( stunServerAddr, &fd2Addr );

	clog << "Opened a stun socket at " << fd2Addr << endl;
	
	StunAddress fd3Addr;

	int fd3,fd4;

	bool ok = stunOpenSocketPair(stunServerAddr,
				     &fd3Addr,
				     &fd3,
				     &fd4 );
	if ( ok )
	{
	    clog << "Opened a stun socket pair at " << fd3Addr << endl;
	    close(fd3);
	    close(fd4);
	}
	else
	{
	    clog << "Opened a stun socket pair FAILED" << endl;
	}
	
	close(fd1);
	close(fd2);

    }
    else
    {
	stunTest( stunServerAddr,testNum,verbose );
    }

#endif
    return 0;
}


// Local Variables:
// mode:c++
// c-file-style:"ellemtel"
// c-file-offsets:((case-label . +))
// indent-tabs-mode:nil
// End:
